#!/bin/bash

q() {
    mysql -Ns -uadmin -p$(cat /etc/psa/.psa.shadow) -Dpsa -e"$1"
}

plesk_version() {
    awk -F. '{print $1$2}' /usr/local/psa/version
}

ver=`plesk_version`

if [ -z "$ver" ] ; then
    echo "FATAL: cannot determine version of Plesk -- file /usr/local/psa/version does not exist or empty."
    exit 1
elif [ $ver -lt 104 ] ; then
    echo "FATAL: Plesk version is older than 10.4.4."
    exit 1
fi

HTTPD_VHOSTS_D=`grep ^HTTPD_VHOSTS_D /etc/psa/psa.conf | awk '{print $2}'`
reconfigure_flag=0

q "select d.name, d.cl_id, dp.val, h.www_root, s.login from domains d left join dom_param dp on (dp.dom_id = d.id and dp.param = 'cgi_bin_mode') left join hosting h on h.dom_id = d.id left join sys_users s on s.id = h.sys_user_id where d.htype = 'vrt_hst'" | \
while read domain cl_id cgi_bin_mode www_root sys_user ; do
    if [ "$www_root" = "NULL" ] ; then
        # no physical hosting definition in database for this domain.
        continue
    fi

    if [ "$cgi_bin_mode" = "www-root" ] ; then
        cgi_bin_path=$www_root/cgi-bin 
    else
        cgi_bin_path=$HTTPD_VHOSTS_D/$domain/cgi-bin
    fi

    if [ ! -d $HTTPD_VHOSTS_D/$domain ] ; then
        echo -n "$domain: creating basic virtual host structure: "
        /usr/local/psa/admin/sbin/skelmng --setup-skel --vhost-name $domain --id $cl_id --www-root=$www_root --cgi-bin-path=$cgi_bin_path >/dev/null && echo "ok"
    fi

    if [ ! -d $HTTPD_VHOSTS_D/$domain/conf ] ; then
        reconfigure_flag=1
        echo -n "$domain: creating 'conf' directory: "
        mkdir -p $HTTPD_VHOSTS_D/$domain/conf
        chmod 750 $HTTPD_VHOSTS_D/$domain/conf
        chown root:psaserv $HTTPD_VHOSTS_D/$domain/conf
        echo "ok"
    fi

    if [ ! -d $HTTPD_VHOSTS_D/$domain/statistics/logs ] ; then
        echo -n "$domain: creating 'logs' directory: "

        mkdir -p $HTTPD_VHOSTS_D/$domain/statistics/logs
        chown $sys_user:psaserv $HTTPD_VHOSTS_D/$domain/statistics/logs
        chmod 550 $HTTPD_VHOSTS_D/$domain/statistics/logs

        touch $HTTPD_VHOSTS_D/$domain/statistics/logs/error_log

        for log in access proxy webmail_access ; do 
            touch $HTTPD_VHOSTS_D/$domain/statistics/logs/${log}_log
            touch $HTTPD_VHOSTS_D/$domain/statistics/logs/${log}_ssl_log
        done

        echo "ok"
    fi
done

if [ $reconfigure_flag -gt 0 ] ; then
    echo -e "\nRebuilding Apache configuration: "

    q "DELETE FROM Configurations"

    sed -i /Include/d /usr/local/psa/admin/conf/ip_vhosts_bootstrap.conf 2>/dev/null
    sed -i /Include/d /usr/local/psa/admin/conf/vhosts_bootstrap.conf 2>/dev/null
    sed -i /include/d /usr/local/psa/admin/conf/nginx_vhosts_bootstrap.conf 2>/dev/null
    sed -i /Include/d /usr/local/psa/admin/conf/webmail_horde_bootstrap.conf 2>/dev/null
    sed -i /Include/d /usr/local/psa/admin/conf/webmail_atmail_bootstrap.conf 2>/dev/null
    sed -i /Include/d /etc/httpd/conf.d/zz010_psa_httpd.conf 2>/dev/null
    sed -i /Include/d /etc/apache2/conf.d/zz010_psa_httpd.conf 2>/dev/null
    rm -f /usr/local/psa/admin/conf/generated/*

    /usr/local/psa/admin/bin/httpdmng --reconfigure-all && echo "ok"
fi
